'use strict';

var _ = require('lodash'),
    _str = require('underscore.string'),
    shell = require('shelljs'),
    fs = require('fs'),
    debug = require('debug')('rsa-translator'),
    async = require('async');

_.mixin(_str);

var getRandomString = function() {
    var text = '';
    var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for (var i=0; i < 10; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
};

module.exports = {

    'generateKeyPair': function(options, fn) {

        debug('Generating key pair');

        options = options || {};
        _.defaults(options, {
            'strength': 2048
        });

        async.waterfall([
            function(cb) {
                var cmd = _.sprintf('openssl genrsa %s', options.strength);
                shell.exec(cmd, {
                    'async': true,
                    'silent': true
                }, function(code, output) {
                    if (code !== 0) {
                        return cb(output);
                    }
                    var tmp = output.split('-----');
                    if (tmp[1] !== 'BEGIN RSA PRIVATE KEY') {
                        var err = 'Error generating private key: ' + output;
                        debug(err);
                        return cb(err);
                    }
                    tmp[0] = '';
                    tmp = tmp.join('-----');
                    debug('Private key generated');
                    return cb(null, tmp);
                });
            },
            function(priv, cb) {

                var getPrivateFile = function(cb) {

                    debug('Getting reference to private key file');

                    async.forever(
                        function(next) {
                            var now = new Date().getTime(),
                                dest = '/tmp/' + getRandomString() + '.' + now;
                            fs.exists(dest, function(exists) {
                                if (!exists) {
                                    debug('Private key file found:', dest);
                                    return cb(null, dest);
                                } else {
                                    next(null);
                                }
                            });
                        },
                        function(err) {
                            return cb(err);
                        }
                    );

                };

                getPrivateFile(function(err, dest) {
                    if (err) {
                        debug('Unable to get private key file:', err);
                        return cb(err);
                    }
                    debug('Private key file is:', dest);
                    fs.writeFile(dest, priv, function(err) {
                        if (err) {
                            return cb(err);
                        }
                        return cb(null, priv, dest);
                    });
                });

            },
            function(priv, privFile, cb) {
                var cmd = _.sprintf('openssl rsa -in %s -pubout', privFile);
                shell.exec(cmd, {
                    'async': true,
                    'silent': true
                }, function(code, output) {
                    if (code !== 0) {
                        return cb(output);
                    }
                    var tmp = output.split('-----');
                    if (tmp[1] !== 'BEGIN PUBLIC KEY') {
                        return cb('Error generating public key: ' + output);
                    }
                    tmp[0] = '';
                    tmp = tmp.join('-----');
                    fs.unlink(privFile, function(err) {
                        if (err) {
                            return cb(err);
                        }
                        return cb(null, {
                            'private': priv,
                            'public': tmp
                        });
                    });
                });
            }
        ], function(err, result) {
            fn(null, {
                'public': result.public,
                'private': result.private
            });
        });

    },

    'decrypt': function(data, options, cb) {

        options = options || {};
        if (!options.private_key) {
            return cb('`options.private_key` is required');
        }

        async.series({
            'data_file': function(cb) {
                async.forever(
                    function(next) {
                        var now = new Date().getTime(),
                            dest = '/tmp/' + getRandomString() + '.' + now;
                        fs.exists(dest, function(exists) {
                            if (!exists) {
                                fs.writeFile(dest, data, function(err) {
                                    if (err) {
                                        return cb(err);
                                    }
                                    return cb(null, dest);
                                });
                            } else {
                                next(null);
                            }
                        });
                    },
                    function(err) {
                        return cb(err);
                    }
                );
            },
            'key_file': function(cb) {
                async.forever(
                    function(next) {
                        var now = new Date().getTime(),
                            dest = '/tmp/' + getRandomString() + '.' + now;
                        fs.exists(dest, function(exists) {
                            if (!exists) {
                                fs.writeFile(dest, options.private_key, function(err) {
                                    if (err) {
                                        return cb(err);
                                    }
                                    return cb(null, dest);
                                });
                            } else {
                                next(null);
                            }
                        });
                    },
                    function(err) {
                        return cb(err);
                    }
                );
            }
        }, function(err, result) {
            if (err) {
                return cb(err);
            }
            var cmd = _.sprintf('openssl rsautl -decrypt -inkey %s -in %s', result.key_file, result.data_file);
            shell.exec(cmd, {
                'async': true,
                'silent': true
            }, function(code, output) {

                if (code !== 0) {
                    return cb(output);
                }

                async.parallel([
                    function(cb) {
                        fs.unlink(result.key_file, function(err) {
                            return cb(err);
                        });
                    },
                    function(cb) {
                        fs.unlink(result.data_file, function(err) {
                            return cb(err);
                        });
                    }
                ], function(err, result) {
                    if (err) {
                        return cb(err);
                    }
                    return cb(null, output);
                });

            });
        });

    },

    'encrypt': function(data, options, cb) {

        options = options || {};
        if (!options.public_key) {
            return cb('`options.public_key` is required');
        }

        async.series({
            'data_file': function(cb) {
                async.forever(
                    function(next) {
                        var now = new Date().getTime(),
                            dest = '/tmp/' + getRandomString() + '.' + now;
                        fs.exists(dest, function(exists) {
                            if (!exists) {
                                fs.writeFile(dest, data, function(err) {
                                    if (err) {
                                        return cb(err);
                                    }
                                    return cb(null, dest);
                                });
                            } else {
                                next(null);
                            }
                        });
                    },
                    function(err) {
                        return cb(err);
                    }
                );
            },
            'key_file': function(cb) {
                async.forever(
                    function(next) {
                        var now = new Date().getTime(),
                            dest = '/tmp/' + getRandomString() + '.' + now;
                        fs.exists(dest, function(exists) {
                            if (!exists) {
                                fs.writeFile(dest, options.public_key, function(err) {
                                    if (err) {
                                        return cb(err);
                                    }
                                    return cb(null, dest);
                                });
                            } else {
                                next(null);
                            }
                        });
                    },
                    function(err) {
                        return cb(err);
                    }
                );
            }
        }, function(err, result) {
            if (err) {
                return cb(err);
            }
            var cmd = _.sprintf('openssl rsautl -encrypt -inkey %s -pubin -in %s', result.key_file, result.data_file);
            shell.exec(cmd, {
                'async': true,
                'silent': true
            }, function(code, output) {

                if (code !== 0) {
                    return cb(output);
                }

                async.parallel([
                    function(cb) {
                        fs.unlink(result.key_file, function(err) {
                            return cb(err);
                        });
                    },
                    function(cb) {
                        fs.unlink(result.data_file, function(err) {
                            return cb(err);
                        });
                    }
                ], function(err, result) {
                    if (err) {
                        return cb(err);
                    }
                    return cb(null, output);
                });

            });
        });

    }

};

